Type: Package
Package: connectapi
Title: Utilities for Interacting with the 'Posit Connect' Server API
Version: 0.12.1
Authors@R: c(
    person(given = "Kara",
           family = "Woo",
           role = c("aut", "cre"),
           email = "kara.woo@posit.co"),
    person(given = "Toph",
           family = "Allen",
           role = c("aut"),
           email = "toph@posit.co"),
    person(given = "Neal",
           family = "Richardson",
           role = c("aut")),
    person(given = "Sean",
           family = "Lopp",
           role = c("aut")),
    person(given = "Cole",
           family = "Arendt",
           role = c("aut")),
    person(given = "Posit, PBC",
           role = c("cph", "fnd")))
Description: Provides a helpful 'R6' class and methods for interacting with
    the 'Posit Connect' Server API along with some meaningful utility functions
    for regular tasks. API documentation varies by 'Posit Connect' installation
    and version, but the latest documentation is also hosted publicly at
    <https://docs.posit.co/connect/api/>.
License: MIT + file LICENSE
URL: https://posit-dev.github.io/connectapi/,
        https://github.com/posit-dev/connectapi
BugReports: https://github.com/posit-dev/connectapi/issues
Imports: bit64, fs, glue, httr, mime, jsonlite, lifecycle, magrittr,
        purrr, R6, rlang (>= 0.4.2), tibble, uuid, vctrs (>= 0.3.0),
        base64enc
Suggests: covr, dbplyr, dplyr, ggplot2, gridExtra, httptest, knitr,
        lubridate, progress, rmarkdown, rprojroot, rsconnect, spelling,
        testthat, tidyr, webshot2, withr
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
Config/testthat/edition: 3
Collate: 'audits.R' 'browse.R' 'connect.R' 'connectapi-package.R'
        'connectapi.R' 'content.R' 'deploy.R' 'get.R' 'git.R'
        'groups.R' 'integrations.R' 'lazy.R' 'page.R' 'parse.R'
        'promote.R' 'ptype.R' 'remote.R' 'runtime-caches.R'
        'schedule.R' 'tags.R' 'utils.R' 'thumbnail.R' 'user.R'
        'utils-pipe.R' 'variant.R'
NeedsCompilation: no
Packaged: 2026-04-29 19:15:15 UTC; kara
Author: Kara Woo [aut, cre],
  Toph Allen [aut],
  Neal Richardson [aut],
  Sean Lopp [aut],
  Cole Arendt [aut],
  Posit, PBC [cph, fnd]
Maintainer: Kara Woo <kara.woo@posit.co>
Repository: CRAN
Date/Publication: 2026-04-29 20:00:03 UTC
Built: R 4.6.0; ; 2026-04-29 20:51:59 UTC; unix
