# Generated by roxygen2: do not edit by hand

S3method("$",connect_tag_tree)
S3method("[",connect_tag_tree)
S3method("[[",connect_tag_tree)
S3method(api_build,op_base_connect)
S3method(api_build,op_head)
S3method(as.data.frame,connect_content_list)
S3method(as.data.frame,connect_integration_list)
S3method(as.data.frame,connect_list_hits)
S3method(as.data.frame,tbl_connect)
S3method(as_tibble,connect_content_list)
S3method(as_tibble,connect_integration_list)
S3method(as_tibble,connect_list_hits)
S3method(connect_vars,op_base)
S3method(connect_vars,op_single)
S3method(connect_vars,tbl_connect)
S3method(dim,tbl_connect)
S3method(dimnames,tbl_connect)
S3method(get_integrations,Connect)
S3method(get_integrations,Content)
S3method(get_integrations,default)
S3method(head,tbl_connect)
S3method(print,connect_integration)
S3method(print,connect_tag_tree)
S3method(print,tbl_connect)
export("%>%")
export(Bundle)
export(Connect)
export(Content)
export(ContentTask)
export(Environment)
export(Task)
export(Vanity)
export(Variant)
export(VariantSchedule)
export(audit_access_open)
export(audit_r_versions)
export(audit_runas)
export(browse_api_docs)
export(browse_connect)
export(browse_dashboard)
export(browse_solo)
export(bundle_dir)
export(bundle_path)
export(bundle_static)
export(connect)
export(content_add_group)
export(content_add_user)
export(content_delete)
export(content_delete_group)
export(content_delete_user)
export(content_item)
export(content_list_by_tag)
export(content_list_guid_has_access)
export(content_list_with_permissions)
export(content_render)
export(content_restart)
export(content_title)
export(content_update)
export(content_update_access_type)
export(content_update_owner)
export(create_integration)
export(create_random_name)
export(create_tag)
export(create_tag_tree)
export(dashboard_url)
export(delete_bundle)
export(delete_integration)
export(delete_runtime_cache)
export(delete_tag)
export(delete_thumbnail)
export(delete_vanity_url)
export(deploy)
export(deploy_current)
export(deploy_repo)
export(deploy_repo_enable)
export(deploy_repo_update)
export(download_bundle)
export(filter_tag_tree_chr)
export(filter_tag_tree_id)
export(get_associations)
export(get_audit_logs)
export(get_aws_content_credentials)
export(get_aws_credentials)
export(get_bundles)
export(get_content)
export(get_content_packages)
export(get_content_permissions)
export(get_content_tags)
export(get_environment)
export(get_group_content)
export(get_group_members)
export(get_group_permission)
export(get_groups)
export(get_integration)
export(get_integrations)
export(get_job_list)
export(get_jobs)
export(get_log)
export(get_my_permission)
export(get_oauth_content_credentials)
export(get_oauth_credentials)
export(get_packages)
export(get_procs)
export(get_runtime_caches)
export(get_runtimes)
export(get_tag_data)
export(get_tags)
export(get_thumbnail)
export(get_timezones)
export(get_usage)
export(get_usage_shiny)
export(get_usage_static)
export(get_user_permission)
export(get_users)
export(get_vanity_url)
export(get_vanity_urls)
export(get_variant)
export(get_variant_default)
export(get_variant_renderings)
export(get_variant_schedule)
export(get_variants)
export(groups_create_remote)
export(has_thumbnail)
export(lock_content)
export(page_cursor)
export(page_offset)
export(poll_task)
export(promote)
export(repo_check_account)
export(repo_check_branches)
export(repo_check_branches_ref)
export(repo_check_manifest_dirs)
export(schedule_describe)
export(search_content)
export(set_content_tag_tree)
export(set_content_tags)
export(set_environment_all)
export(set_environment_new)
export(set_environment_remove)
export(set_integrations)
export(set_run_as)
export(set_schedule)
export(set_schedule_day)
export(set_schedule_dayofmonth)
export(set_schedule_dayofweek)
export(set_schedule_dayweekofmonth)
export(set_schedule_hour)
export(set_schedule_minute)
export(set_schedule_remove)
export(set_schedule_semimonth)
export(set_schedule_week)
export(set_schedule_weekday)
export(set_schedule_year)
export(set_thumbnail)
export(set_vanity_url)
export(swap_vanity_urls)
export(tbl_connect)
export(terminate_jobs)
export(unlock_content)
export(update_integration)
export(user_guid_from_username)
export(users_create_remote)
export(vanity_is_available)
export(variant_render)
export(verify_content_name)
importFrom(lifecycle,deprecate_warn)
importFrom(lifecycle,deprecated)
importFrom(magrittr,"%>%")
importFrom(rlang,"%||%")
importFrom(rlang,":=")
importFrom(rlang,arg_match)
importFrom(rlang,is_string)
importFrom(tibble,as_tibble)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,compareVersion)
importFrom(utils,head)
importFrom(utils,untar)
importFrom(uuid,UUIDgenerate)
