structure(
  list(
    url = "__api__/v1/content/8f37d6e0/jobs/waaTO7v75I84S1hQ",
    status_code = 409L,
    headers = structure(
      list(`content-type` = "application/json"),
      class = c("insensitive", "list")
    ),
    content = charToRaw(
      "{\"code\":163,\"error\":\"The specified job cannot be terminated because it is not active\",\"payload\":null}"
    )
  ),
  class = "response"
)
