structure(
  list(
    url = "__api__/applications/23456789/image",
    status_code = 404L,
    content = list(
      code = 4,
      error = "The requested object does not exist.",
      payload = NULL
    )
  ),
  class = "response"
)
