structure(
  list(
    url = "__api__/v1/applications/34567890/thumbnail",
    status_code = 404L,
    content = list(
      code = 17,
      error = "The requested item could not be removed because it does not exist.",
      payload = NULL
    )
  ),
  class = "response"
)
