structure(
  list(
    url = "content/01234567/__thumbnail__",
    status_code = 200L,
    headers = structure(list(
      `content-type` = "image/jpeg"
    )),
    content = as.raw(c(
      0x4e,
      0x41
    ))
  ),
  class = "response"
)
