structure(
  list(
    request <- list(url = "__api__/v1/system/caches/runtime"),
    url = "__api__/v1/system/caches/runtime",
    status_code = 404L,
    content = charToRaw(
      "{\"code\":4,\"error\":\"Cache does not exist\",\"payload\":null}"
    )
  ),
  class = "response"
)
