structure(
  list(
    url = "__api__/v1/content/c3426b0b/vanity",
    status_code = 403L,
    content = charToRaw(
      "{\"code\":22,\"error\":\"You don't have permission to perform this operation.\",\"payload\":null}"
    )
  ),
  class = "response"
)
