structure(
  list(
    url = "__api__/v1/content/01234567/thumbnail",
    status_code = 404L,
    content = charToRaw("404 page not found"),
    headers = structure(
      list(`content-type` = "text/plain; charset=utf-8"),
      class = "insensitive"
    )
  ),
  class = "response"
)
