# TODO: This testing requires an LDAP server
# TODO: use the same underlying implementation somehow...

test_that("users_create_remote errors if prefix returns zero matches", {
  skip("not implemented")
})
test_that("users_create_remote errors if prefix returns more than one matches", {
  skip("not implemented")
})
test_that("users_create_remote errors if expect is greater than one", {
  skip("not implemented")
})
test_that("users_create_remote works if match already exists", {
  # should message
  skip("not implemented")
})
test_that("users_create_remote creates match if one is found", {
  skip("not implemented")
})
test_that("users_create_remote creates multiples if multiple are found", {
  skip("not implemented")
})
test_that("users_create_remote avoids existing matches if multiple are found", {
  # fails today
  skip("not implemented")
})

test_that("groups_create_remote errors if prefix returns zero matches", {
  skip("not implemented")
})
test_that("groups_create_remote errors if prefix returns more than one matches", {
  skip("not implemented")
})
test_that("groups_create_remote errors if expect is greater than one", {
  skip("not implemented")
})
test_that("groups_create_remote works if match already exists", {
  # should message
  skip("not implemented")
})
test_that("groups_create_remote creates match if one is found", {
  skip("not implemented")
})
test_that("groups_create_remote creates multiples if multiple are found", {
  skip("not implemented")
})
test_that("groups_create_remote avoids existing matches if multiple are found", {
  # fails today
  skip("not implemented")
})
