# we can retrieve a content item

    Code
      print(item)
    Output
      Posit Connect Content: 
        Content GUID: f2f37341-e21d-3d80-c698-a935ad614066
        Content URL: https://connect.example/connect/#/apps/f2f37341-e21d-3d80-c698-a935ad614066
        Content Title: Performance Data
      
      content_item(client, guid = "f2f37341-e21d-3d80-c698-a935ad614066")
      

